/* $Id: pollers_test.c,v 1.1.1.1 2006/07/12 01:52:26 wlmarque Exp $ */
#include <sys/types.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <net/if.h>
#include <net/pfvar.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <err.h>
#include <errno.h>
#include <fcntl.h>
#include <poll.h>
#include <unistd.h>
#include <pcap.h>
#include <assert.h>
#include <stdlib.h>
#include <pthread.h>

#include "service.h"
#include "vsvc.h"
#include "pollers.h"

int test_service_polltcp(struct service_t *);

int test_service_polltcp(struct service_t *s) {
	warnx("testing service_polltcp");
	lock_service(s);
	setservice_addpolltype(s, SVCPOLL_TCP);
	unlock_service(s);

	warnx(" with a known good port (10.0.0.36:22)");
	setservice_inaddr(s, "10.0.0.36");
	setservice_port(s, 22);
	setservice_down(s);
	setservice_active(s);
	
	assert(service_polltcp(s) == 0);
	assert((getservice_status(s) & SVCSTATUS_UP) == SVCSTATUS_UP);
	print_service(s);

	warnx(" with a known good port (10.0.0.2:22)");
	setservice_inaddr(s, "10.0.0.2");
	setservice_port(s, 22);
	setservice_down(s);
	setservice_active(s);
	
	assert(service_polltcp(s) == 0);
	assert((getservice_status(s) & SVCSTATUS_UP) == SVCSTATUS_UP);
	print_service(s);
	
	warnx(" with a known so-so port (10.0.0.2:21)");
	setservice_inaddr(s, "10.0.0.2");
	setservice_port(s, 21);
	setservice_down(s);
	setservice_active(s);
	
	assert(service_polltcp(s) == 0);
	assert((getservice_status(s) & SVCSTATUS_UP) == SVCSTATUS_UP);
	print_service(s);

	warnx(" with a known bad port (10.0.0.36:8)");
	setservice_inaddr(s, "10.0.0.36");
	setservice_port(s, 8);
	setservice_down(s);
	setservice_active(s);

	assert(service_polltcp(s) == 0);
	assert((getservice_status(s) & SVCSTATUS_UP) == 0);
	print_service(s);

	/* save time
	warnx(" with a known down host (10.0.0.1:8)");
	setservice_inaddr(s, "10.0.0.1");
	setservice_port(s, 8);

	assert(service_polltcp(s) == 0);
	assert((getservice_status(s) & SVCSTATUS_UP) == 0);
	print_service(s);
	*/

	warnx("service_polltcp ok");
	return(0);
}

int test_service_pollhttp(struct service_t *s) {
	warnx("testing service_polllhttp against www.google.com");
	warnx(" with a known up host (216.239.37.99:80)");
	setservice_inaddr(s, "216.239.37.99");
	setservice_port(s, 80);
	setservice_httpget(s, "/", "200 OK");

	assert(service_pollhttp(s) == 0);
	assert(getservice_status(s) & SVCSTATUS_UP);
	print_service(s);

	warnx(" with a known up host but a bad URL (216.239.37.99:80)");
	setservice_inaddr(s, "216.239.37.99");
	setservice_port(s, 80);
	setservice_httpget(s, "/website/index.jhtml", "200 OK");

	assert(service_pollhttp(s) == 0);
	assert((getservice_status(s) & SVCSTATUS_UP) == 0);
	print_service(s);

	warnx("service_pollhttp ok");

	return(0);
}


int main(void) {
	struct service_t *s;

	s = malloc(sizeof(struct service_t));

	init_service(s);

	assert(test_service_polltcp(s) == 0);
	assert(test_service_pollhttp(s) == 0);

	return(0);
}
