/* $Id: service_test.c,v 1.1.1.1 2006/07/12 01:52:26 wlmarque Exp $ */
#include <assert.h>
#include <pthread.h>
#include <sys/queue.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <stdio.h>
#include <time.h>

#include "service.h"

int test_init_service(struct service_t *);
int test_lock_service(struct service_t *);
int test_unlock_service(struct service_t *);
int test_setservice_status(struct service_t *);
int test_setservice_active(struct service_t *);
int test_setservice_inactive(struct service_t *);
int test_setservice_up(struct service_t *);
int test_setservice_down(struct service_t *);
int test_setservice_polltype(struct service_t *);
int test_setservice_addpolltype(struct service_t *);
int test_setservice_rmpolltype(struct service_t *);
int test_getservice_polltype(struct service_t *);
int test_getservice_status(struct service_t *);
int test_setservice_httpget(struct service_t *); 
int test_setservice_httphead(struct service_t *);
int test_setservice_ping(struct service_t *);
int test_getservice_httpget(struct service_t *);
int test_setservice_inaddr(struct service_t *);
int test_getservice_inaddr(struct service_t *);
int test_setservice_port(struct service_t *);
int test_getservice_port(struct service_t *);

/* int test_getservice_httppost(struct service_t *); */

int test_init_service(struct service_t *service) {
	printf("Testing init_service\n");
	fflush(stdout);

	assert(init_service(service) == 0);
	assert(test_lock_service(service) == 0);
	assert(getservice_polltype(service) == 0x0);
	assert(getservice_status(service) == 0x0);
	assert(test_unlock_service(service) == 0);

	return(0);
}

int test_setservice_status(struct service_t *service) {
	printf("Testing setservice_status\n");
	fflush(stdout);
	assert(test_lock_service(service) == 0);
	assert(setservice_status(service, SVCSTATUS_ACTIVE|SVCSTATUS_UP) == \
			(SVCSTATUS_ACTIVE|SVCSTATUS_UP));
	assert((SVCSTATUS_ACTIVE|SVCSTATUS_UP) == getservice_status(service));
	assert(test_unlock_service(service) == 0);
	return(0);
}

int test_lock_service(struct service_t *service) {
	printf("Testing lock_service\n");
	if (lock_service(service)) {
		return (1);
	}
	else {
		return(0);
	}
}

int test_unlock_service(struct service_t *service) {
	printf("Testing unlock_service\n");
	if (unlock_service(service)) {
		return (1);
	}
	else {
		return(0);
	}
}

int test_setservice_active(struct service_t *service) {
	printf("Testing setservice_active\n");
	fflush(stdout);
	assert(test_lock_service(service) == 0);

	assert(setservice_active(service) & SVCSTATUS_ACTIVE);
	assert(setservice_status(service, SVCSTATUS_ACTIVE) == \
			SVCSTATUS_ACTIVE);

	assert(test_unlock_service(service) == 0);
	return(0);
}

int test_setservice_inactive(struct service_t *service) {
	printf("Testing setservice_inactive\n");
	fflush(stdout);
	assert(test_lock_service(service) == 0);

	assert((setservice_inactive(service) & SVCSTATUS_ACTIVE) == 0);
	setservice_status(service, 0xFFFF);
	setservice_inactive(service);
	assert(!(getservice_status(service) & SVCSTATUS_ACTIVE));

	assert(test_unlock_service(service) == 0);
	return(0);
}

int test_setservice_up(struct service_t *service) {
	printf("Testing setservice_up\n");
	fflush(stdout);
	assert(test_lock_service(service) == 0);

	assert(setservice_up(service) & SVCSTATUS_UP);
	setservice_status(service, 0x0000);
	assert(setservice_up(service) == SVCSTATUS_UP);

	assert(test_unlock_service(service) == 0);
	return(0);
}

int test_setservice_down(struct service_t *service) {
	printf("Testing setservice_down\n");
	fflush(stdout);
	assert(test_lock_service(service) == 0);

	assert(!(setservice_down(service) & SVCSTATUS_UP)); /* FIXME */
	printf("original %x setservice_down(service) gives %x and SVCSTATUS_UP is %x\n",
			(unsigned int) getservice_status(service), (unsigned int) setservice_down(service),
			SVCSTATUS_UP);
	/* assert((setservice_down(service) & SVCSTATUS_UP) == 0); */
	setservice_status(service, 0x0000);
	assert(!(setservice_up(service) == 0));

	assert(test_unlock_service(service) == 0);
	return(0);
}

int test_getservice_polltype(struct service_t *service) {
	printf("Testing getservice_polltype\n");
	fflush(stdout);
	assert(test_lock_service(service) == 0);

	assert(setservice_polltype(service, 0x0) == 0);
	assert(getservice_polltype(service) == 0);

	assert(test_unlock_service(service) == 0);
	return(0);
}

int test_setservice_polltype(struct service_t *service) {
	printf("Testing setservice_polltype\n");
	fflush(stdout);
	assert(test_lock_service(service) == 0);

	assert(setservice_polltype(service, \
		SVCPOLL_PING|SVCPOLL_TCP|SVCPOLL_HTTPHEAD|SVCPOLL_EXPECT) == \
		(SVCPOLL_PING|SVCPOLL_TCP|SVCPOLL_HTTPHEAD|SVCPOLL_EXPECT));
	assert(setservice_polltype(service, 0x0) == 0);

	assert(test_unlock_service(service) == 0);
	return(0);
}

int test_setservice_addpolltype(struct service_t *service) {
	printf("Testing setservice_addpolltype\n");
	fflush(stdout);
	assert(test_lock_service(service) == 0);

	assert(setservice_polltype(service, 0x0) == 0);
	assert(setservice_addpolltype(service, SVCPOLL_PING) == SVCPOLL_PING);
	assert(setservice_addpolltype(service, SVCPOLL_TCP) == \
			(SVCPOLL_PING|SVCPOLL_TCP));
	assert(setservice_addpolltype(service, SVCPOLL_HTTPHEAD) == \
			(SVCPOLL_PING|SVCPOLL_TCP|SVCPOLL_HTTPHEAD));
	assert(setservice_addpolltype(service, SVCPOLL_EXPECT) == \
		(SVCPOLL_PING|SVCPOLL_TCP|SVCPOLL_HTTPHEAD|SVCPOLL_EXPECT));

	assert(setservice_addpolltype(service, \
		SVCPOLL_PING|SVCPOLL_TCP|SVCPOLL_HTTPHEAD|SVCPOLL_EXPECT) == \
		(SVCPOLL_PING|SVCPOLL_TCP|SVCPOLL_HTTPHEAD|SVCPOLL_EXPECT));

	assert(test_unlock_service(service) == 0);
	return(0);
}

int test_setservice_rmpolltype(struct service_t *service) {
	printf("Testing setservice_rmpolltype\n");
	fflush(stdout);
	assert(test_lock_service(service) == 0);

	assert(setservice_polltype(service, 
		SVCPOLL_PING|SVCPOLL_TCP|SVCPOLL_HTTPHEAD|SVCPOLL_EXPECT) ==
		(SVCPOLL_PING|SVCPOLL_TCP|SVCPOLL_HTTPHEAD|SVCPOLL_EXPECT));
	assert(setservice_rmpolltype(service, SVCPOLL_PING) == \
		(SVCPOLL_TCP|SVCPOLL_HTTPHEAD|SVCPOLL_EXPECT));
	assert(setservice_rmpolltype(service, SVCPOLL_HTTPHEAD) == \
		(SVCPOLL_TCP|SVCPOLL_EXPECT));
	assert(setservice_rmpolltype(service, SVCPOLL_TCP) == SVCPOLL_EXPECT);
	assert(setservice_rmpolltype(service, SVCPOLL_EXPECT) == 0);

	assert(test_unlock_service(service) == 0);
	return(0);
}

int test_getservice_status(struct service_t *service) {
	printf("Testing getservice_status\n");
	fflush(stdout);
	assert(test_lock_service(service) == 0);

	assert(getservice_status(service) == (
			setservice_status(service,
				getservice_status(service))));

	assert(test_unlock_service(service) == 0);
	return(0);
}

int test_setservice_httpget(struct service_t *service) {
	printf("Testing setservice_httpget\n");
	fflush(stdout);
	assert(test_lock_service(service) == 0);

	assert(setservice_polltype(service, 0x0) == 0);
	assert(setservice_httpget(service,
			"http://www.christianity.com/test",
			"HTTP/1.0 200 OK") == 0);
	assert(getservice_polltype(service) & SVCPOLL_HTTPGET);
	assert(getservice_polltype(service) & SVCPOLL_EXPECT);

	assert(test_unlock_service(service) == 0);
	return(0);
}

int test_setservice_ping(struct service_t *service) {
	printf("Testing setservice_ping\n");
	fflush(stdout);
	assert(test_lock_service(service) == 0);

	assert(setservice_polltype(service, 0x0) == 0);
	assert(setservice_ping(service) == 0);
	assert(getservice_polltype(service) & SVCPOLL_PING);

	assert(test_unlock_service(service) == 0);
	return(0);
}

/*
int test_setservice_httppost(struct service_t *service) {
	printf("Testing setservice_httppost\n");
	fflush(stdout);
	assert(test_lock_service(service) == 0);

	assert(setservice_polltype(service, 0x0) == 0);
	assert(setservice_httppost(service,
			"http://www.christianity.com/test",
			"HTTP/1.0 200 OK") == 0);
	assert(getservice_polltype(service) & SVCPOLL_HTTPPOST);

	assert(test_unlock_service(service) == 0);
	return(0);
}
*/

int test_setservice_httphead(struct service_t *service) {
	printf("Testing setservice_httphead\n");
	fflush(stdout);
	assert(test_lock_service(service) == 0);

	assert(setservice_polltype(service, 0x0) == 0);
	assert(setservice_httphead(service,
		"http://www.christianity.com/test",
		"HTTP/1.0 200 OK") == 0);
	assert(getservice_polltype(service) & SVCPOLL_HTTPHEAD);
	assert(getservice_polltype(service) & SVCPOLL_EXPECT);

	assert(test_unlock_service(service) == 0);
	return(0);
}

int test_setservice_inaddr(struct service_t *service) {
	printf("Testing setservice_inaddr\n");
	fflush(stdout);
	assert(test_lock_service(service) == 0);
	assert(setservice_inaddr(service, "127.0.0.1") == 0);
	assert(test_unlock_service(service) == 0);
	return(0);
}

int test_getservice_inaddr(struct service_t *service) {
	printf("Testing getservice_inaddr\n");
	fflush(stdout);
	assert(test_lock_service(service) == 0);
	assert(setservice_inaddr(service, "127.0.0.1") == 0);
	assert(strcmp("127.0.0.1", (char *) inet_ntoa(getservice_inaddr(service))) == 0);
	assert(test_unlock_service(service) == 0);
	return(0);
}

int test_setservice_port(struct service_t *service) {
	printf("Testing setservice_port\n");
	fflush(stdout);
	assert(test_lock_service(service) == 0);
	assert(setservice_port(service, 80) == 0);
	assert (service->addr.sin_port == htons(80));
	assert(test_unlock_service(service) == 0);
	
	return(0);
}

int test_getservice_port(struct service_t *service) {
	printf("Testing getservice_port\n");
	fflush(stdout);
	assert(test_lock_service(service) == 0);
	assert(setservice_port(service, 80) == 0);
	assert(getservice_port(service) == 80);
	assert(test_unlock_service(service) == 0);
	
	return(0);
}

int main(void) {
	struct service_t test_svc;

	assert(0 == test_init_service(&test_svc));
	assert(0 == test_lock_service(&test_svc));
	assert(0 == test_unlock_service(&test_svc));
	assert(0 == test_setservice_status(&test_svc));
	assert(0 == test_setservice_active(&test_svc));
	assert(0 == test_setservice_inactive(&test_svc));
	assert(0 == test_setservice_up(&test_svc));
	assert(0 == test_setservice_down(&test_svc));
	assert(0 == test_setservice_polltype(&test_svc));
	assert(0 == test_setservice_addpolltype(&test_svc));
	assert(0 == test_getservice_polltype(&test_svc));
	assert(0 == test_getservice_status(&test_svc));
	assert(0 == test_setservice_httpget(&test_svc)); 
	assert(0 == test_setservice_httphead(&test_svc));
	assert(0 == test_setservice_ping(&test_svc));
	assert(0 == test_setservice_inaddr(&test_svc));
	assert(0 == test_getservice_inaddr(&test_svc));
	assert(0 == test_setservice_port(&test_svc));
	assert(0 == test_getservice_port(&test_svc));

	return(0);
}
