/* $Id: vsvc_rules_test.c,v 1.1.1.1 2006/07/12 01:52:26 wlmarque Exp $ */
#include <sys/types.h>
#include <netinet/in.h>
#include <net/pfvar.h>
#include <assert.h>
#include <stdio.h>
#include <err.h>
#include <string.h>
#include <unistd.h>

#include "globals.h"
#include "pfctl/pfctl.h"
#include "pfctl/pfctl_parser.h"
#include "vsvc.h"
#include "vsvc_rules.h"


int test_vsvc_rulesetinit();
int test_vsvc_rulefetch();
int test_vsvc_ruleadd(struct vsvc_t *, struct pf_rule *);
int test_vsvc_ruledelete(struct vsvc_t *);
int test_vsvc_rulecreate(struct vsvc_t *);
int test_vsvc_rulesetdestroy(struct pf_rule *);

int test_vsvc_pfctlstart();
int test_vsvc_pfctlstop();
int test_vsvc_pfctllock();
int test_vsvc_pfctlunlock();
int test_vsvc_pfctlgetstatus();

int test_vsvc_ruleupdate(struct pf_rule *);
int test_vsvc_poolupdate(struct vsvc_t *);

int test_vsvc_rulesetinit() {
	int fd;
	struct pfioc_rule pr;

	warnx("Testing vsvc_rulesetinit");
	assert(vsvc_pfctlgetstatus() & PFCTLSTATUS_READY);
	assert(vsvc_rulesetinit() == 0);
	fd = open("/dev/pf", O_RDWR, 0);
	assert(fd >= 0);
	
	memset(&pr, 0, sizeof(pr));
	pr.action = PF_CHANGE_GET_TICKET;
	pr.rule.action = PF_RDR;
	strlcpy(pr.anchor, anchorname, PF_ANCHOR_NAME_SIZE);
	strlcpy(pr.ruleset, rulesetname, PF_RULESET_NAME_SIZE);

	if (ioctl(dev, DIOCGETRULESETS, &pr))
		err("DIOCGETRULESETS");

	assert(pr.nr != 0);

	close(fd);

	return(0);
}

int test_vsvc_rulefetch() {
	int fd;
	pf_rule p;

	fd = open("/dev/pf", O_RDWR, 0);
	assert(fd >= 0);

	pr.action = PF_CHANGE_GET_TICKET;
	pr.rule.action = PF_RDR;
	strlcpy(pr.anchor, anchorname, PF_ANCHOR_NAME_SIZE);
	strlcpy(pr.ruleset, rulesetname, PF_RULESET_NAME_SIZE);

        if (ioctl(dev, DIOCGETRULESETS, &pr))
		err("DIOCGETRULESETS");

	/* XXX don't do anything here */

	close(fd);
	return(0);
}

int test_vsvc_pfctlstart() {
	warnx("Testing vsvc_pfctlstart");
	assert(vsvc_pfctlstart() == 0);
	assert(pthread_mutex_trylock(&pfcontrol.lock) == 0);
	assert(pthread_mutex_unlock(&pfcontrol.lock) == 0);
	assert((pfcontrol->status & PFCTLSTATUS_READY) == PFCTLSTATUS_READY);

	assert(pfcontrol.ctl.dev != 0);

	return(0);
}

int test_vsvc_pfctlstop() {
	warnx("Testing vsvc_pfctlstop");
	assert(pthread_mutex_trylock(&pfcontrol.lock) == 0);
	assert(pthread_mutex_unlock(&pfcontrol.lock) == 0);
	assert((pfcontrol->status & PFCTLSTATUS_READY) == 0x0);

	assert(pthread_mutex_trylock(&pfcontrol.lock) != 0 && \
		errno == EINVAL);
	return(0);
}

int test_vsvc_pfctllock() {
	warnx("Testing vsvc_pfctllock");
	assert(vsvc_pffctllock() == 0);
	assert(pthread_mutex_trylock(&pfcontrol.lock) != 0);
	assert(errno == EBUSY);

	assert((pfcontrol.status & PFCTLSTATUS_READY) == PFCTLSTATUS_READY);

	return(0);
}

int test_vsvc_pfctlunlock() {
	warnx("Testing vsvc_pfctlunlock");
	assert(vsvc_pffctlunlock() == 0);
	assert(pthread_mutex_trylock(&pfcontrol.lock) == 0);

	assert((pfcontrol.status & PFCTLSTATUS_READY) == PFCTLSTATUS_READY);

	return(0);
}

int test_vsvc_pfctlgetstatus() {
	status_t st;
	warnx("Testing vsvc_pfctlgetstatus");
	st = vsvc_pfctlgetstatus();
	assert(st != 0);
	assert((st | PFCTLSTATUS_READY) == PFCTLSTATUS_READY);
	st &= ~PFCTLSTATUS_READY;
	assert(st == 0x0);

	return(0);
}


int main(void) {

	anchorname = "slb";
	rulesetname = "main";
}
