/* $Id: vsvc_test.c,v 1.1.1.1 2006/07/12 01:52:26 wlmarque Exp $ */
#include <sys/types.h>
#include <sys/queue.h>
#include <sys/socket.h>
#include <net/if.h>
#include <net/pfvar.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <errno.h>
#include <pthread.h>
#include <stdio.h>
#include <assert.h>

#include "globals.h"
#include "service.h"
#include "vsvc.h"

int test_vsvc_init(struct vsvc_t *);
int test_vsvc_lock(struct vsvc_t *);
int test_vsvc_unlock(struct vsvc_t *);
int test_vsvc_getinaddr(struct vsvc_t *);
int test_vsvc_getname(struct vsvc_t *);
int test_vsvc_getport(struct vsvc_t *);
int test_vsvc_setinaddr(struct vsvc_t *);
int test_vsvc_setname(struct vsvc_t *);
int test_vsvc_setport(struct vsvc_t *);

int test_vsvc_setstatus(struct vsvc_t *);
int test_vsvc_getstatus(struct vsvc_t *);
int test_vsvc_setactive(struct vsvc_t *);
int test_vsvc_setinactive(struct vsvc_t *);
int test_vsvc_markup(struct vsvc_t *);
int test_vsvc_markdown(struct vsvc_t *);

int test_vsvc_setstatus(struct vsvc_t *v) {
	status_t status;

	printf("testing vsvc_setstatus\n");
	assert(vsvc_lock(v) == 0);
	status = 0x0;
	status |= VSVCSTATUS_ACTIVE;
	assert(vsvc_setstatus(v, status) | VSVCSTATUS_ACTIVE);
	assert(vsvc_setstatus(v, status) == VSVCSTATUS_ACTIVE);
	status |= VSVCSTATUS_UP;
	assert(vsvc_setstatus(v, status) | VSVCSTATUS_UP);
	assert(vsvc_setstatus(v, status) == (VSVCSTATUS_ACTIVE|VSVCSTATUS_UP));
	assert(vsvc_setstatus(v, 0x0) == 0x0);
	assert(vsvc_unlock(v) == 0);
	return(0);
}

int test_vsvc_getstatus(struct vsvc_t *v) {
	status_t status = VSVCSTATUS_ACTIVE;
	printf("testing vsvc_getstatus\n");
	assert(vsvc_lock(v) == 0);
	assert(vsvc_setstatus(v, status) == VSVCSTATUS_ACTIVE);
	assert(vsvc_getstatus(v) == VSVCSTATUS_ACTIVE);
	assert(vsvc_getstatus(v) == v->status);
	assert(vsvc_unlock(v) == 0);
	return(0);
}

int test_vsvc_setactive(struct vsvc_t *v) {
	printf("testing vsvc_setactive\n");
	assert(vsvc_lock(v) == 0);
	assert(vsvc_setstatus(v, 0) == 0);
	assert(vsvc_setactive(v) | VSVCSTATUS_ACTIVE);
	assert(vsvc_getstatus(v) == VSVCSTATUS_ACTIVE);
	
	assert(vsvc_unlock(v) == 0);
	return(0);
}

int test_vsvc_setinactive(struct vsvc_t *v) {
	printf("testing vsvc_setinactive\n");
	assert(vsvc_lock(v) == 0);
	assert(vsvc_setstatus(v, 0) == 0);
	assert(vsvc_setactive(v) & VSVCSTATUS_ACTIVE);
	assert(vsvc_getstatus(v) == VSVCSTATUS_ACTIVE);
	assert(!(vsvc_setinactive(v) & VSVCSTATUS_ACTIVE));
	assert(vsvc_setstatus(v, VSVCSTATUS_UP|VSVCSTATUS_ACTIVE) == \
		(VSVCSTATUS_UP|VSVCSTATUS_ACTIVE));
	assert(!(vsvc_setinactive(v) & VSVCSTATUS_ACTIVE));
	
	assert(vsvc_unlock(v) == 0);
	return(0);
}

int test_vsvc_markup(struct vsvc_t *v) {
	printf("testing vsvc_markup\n");
	assert(vsvc_lock(v) == 0);

	assert(vsvc_markup(v) & VSVCSTATUS_UP);
	assert(vsvc_getstatus(v) & VSVCSTATUS_UP);
	
	assert(vsvc_unlock(v) == 0);
	return(0);
}

int test_vsvc_markdown(struct vsvc_t *v) {
	printf("testing vsvc_markdown\n");
	assert(vsvc_lock(v) == 0);

	assert(vsvc_setstatus(v, 0xFFFFFFFF) == 0xFFFFFFFF);
	assert(!(vsvc_markdown(v) & VSVCSTATUS_UP));
	assert((vsvc_getstatus(v) & VSVCSTATUS_UP) == 0);
	
	assert(vsvc_unlock(v) == 0);
	return(0);
}

int test_vsvc_init(struct vsvc_t *v) {
	struct vsvc_t z;

	memset(&z, 0, sizeof(z));
	printf("testing vsvc_init\n");
	assert(vsvc_init(v) == 0);
	return(0);
}

int test_vsvc_lock(struct vsvc_t *v) {
	printf("testing vsvc_lock\n");
	assert(vsvc_lock(v) == 0);
	assert((pthread_mutex_trylock(&v->lock) != 0) && (errno != EBUSY));
	return (0);
}

int test_vsvc_unlock(struct vsvc_t *v) {
	printf("testing vsvc_unlock\n");
	assert(vsvc_unlock(v) == 0);
	assert(pthread_mutex_trylock(&v->lock) == 0);
	assert(pthread_mutex_unlock(&v->lock) == 0);
	return(0);
}

int test_vsvc_setinaddr(struct vsvc_t *v) {
	in_addr_t addr;
	char *ia = "127.0.0.1";

	printf("testing vsvc_setinaddr\n");
	assert(vsvc_lock(v) == 0);
	addr = inet_addr(ia);
	assert(vsvc_setinaddr(v, ia) == 0);
	assert(v->addr.sin_addr.s_addr == addr);
	ia = "10.0.0.5";
	addr = inet_addr(ia);
	assert(vsvc_setinaddr(v, ia) == 0);
	assert(v->addr.sin_addr.s_addr == addr);
	assert(vsvc_unlock(v) == 0);

	return(0);
}

int test_vsvc_getinaddr(struct vsvc_t *v) {
	char *ia = "127.0.0.1";
	in_addr_t addr;

	addr = inet_addr(ia);
	printf("testing vsvc_getinaddr\n");
	assert(vsvc_lock(v) == 0);
	assert(vsvc_setinaddr(v, ia) == 0);
	assert((vsvc_getinaddr(v)).s_addr == addr);
	printf("in_addr was definitely %s\n", inet_ntoa(vsvc_getinaddr(v)));
	assert(vsvc_unlock(v) == 0);

	return(0);
}

int test_vsvc_getname(struct vsvc_t *v) {
	char *big = "1234567891234567891234567891234567891234567891234567891234567891234567891234567891234567891234567891234567891234567891234567891234567891234567891234567891234567891234567891234567891234567891234567891234567891234567890000000000000000000000001234567890";
	char *small = "";
	char *justright = "testsvc";
	printf("testing vsvc_getname\n");
	assert(vsvc_lock(v) == 0);

	assert(vsvc_setname(v, big) == 0);
	assert(strlen(vsvc_getname(v)) <= MAXNAMELEN && \
		strlen(vsvc_getname(v)) > 0);
	assert(vsvc_setname(v, small) == 0);
	assert(strlen(vsvc_getname(v)) <= MAXNAMELEN && \
		strlen(vsvc_getname(v)) == 0);
	assert(vsvc_setname(v, justright) == 0);
	assert(strlen(vsvc_getname(v)) == strlen(justright));

	assert(vsvc_unlock(v) == 0);
	return(0);
}

int test_vsvc_setname(struct vsvc_t *v) {
	char big[2000];

	memset(big, 0x0, sizeof(big));
	printf("testing vsvc_setname\n");
	/* zero-length string */
	printf("zero-length string\n");
	assert(vsvc_lock(v) == 0);
	assert(vsvc_setname(v, big) == 0);
	assert(strlen(v->name) == 0);

	/* normal-length string */
	printf("normal-length string\n");
	memset(big, 'A', 50);
	assert(vsvc_setname(v, big) == 0);
	assert(strlen(v->name) == 50);
	memset(big, 0x0, sizeof(big));

	/* one less than max */
	printf("one less than max\n");
	memset(big, 'A', MAXNAMELEN-1);
	assert(vsvc_setname(v, big) == 0);
	assert(strlen(v->name) == MAXNAMELEN-1);
	memset(big, 0x0, sizeof(big));

	/* right at the max */
	printf("right at the max\n");
	memset(big, 'A', MAXNAMELEN);
	assert(vsvc_setname(v, big) == 0);
	assert(strlen(v->name) == MAXNAMELEN);
	memset(big, 0x0, sizeof(big));

	/* one over the max */
	printf("one over the max\n");
	memset(big, 'A', MAXNAMELEN+1);
	assert(vsvc_setname(v, big) == 0);
	assert(strlen(v->name) == MAXNAMELEN);
	memset(big, 0x0, sizeof(big));

	/* lots over the max */
	printf("lots over the max\n");
	memset(big, 'A', 1500);
	assert(vsvc_setname(v, big) == 0);
	assert(strlen(v->name) == MAXNAMELEN);
	memset(big, 0x0, sizeof(big));

	assert(vsvc_unlock(v) == 0);

	return(0);
}

int test_vsvc_setport(struct vsvc_t *v) {
	int port = 443;

	printf("testing vsvc_setport\n");
	assert(vsvc_lock(v) == 0);

	printf("testing setport with good port\n");
	assert(vsvc_setport(v, port) == 0);
	assert(v->addr.sin_port == ntohs((in_port_t) port));

	printf("testing setport with a too-big port\n");
	port = 65537;
	assert(vsvc_setport(v, port) != 0);

	printf("testing setport with a too-small port\n");
	port = 0;
	assert(vsvc_setport(v, port) != 0);
	port = -255;
	assert(vsvc_setport(v, port) != 0);

	assert(vsvc_unlock(v) == 0);
	return(0);
}

int	test_vsvc_getport(struct vsvc_t *v) {
	int port = 80;

	printf("testing vsvc_getport\n");
	assert(vsvc_lock(v) == 0);
	assert(vsvc_setport(v, port) == 0);
	assert(vsvc_getport(v) == 80);
	assert(vsvc_getport(v) == port);
	assert(vsvc_unlock(v) == 0);
	return(0);
} 

int main(void) {
	struct vsvc_t service;

	printf("Ready to test vsvc functions\n");

	assert(test_vsvc_init(&service) == 0);
	assert(test_vsvc_lock(&service) == 0);
	assert(test_vsvc_unlock(&service) == 0);
	assert(test_vsvc_getinaddr(&service) == 0);
	assert(test_vsvc_getname(&service) == 0);
	assert(test_vsvc_setname(&service) == 0);
	assert(test_vsvc_getport(&service) == 0);
	assert(test_vsvc_setinaddr(&service) == 0);
	assert(test_vsvc_setport(&service) == 0);
	assert(test_vsvc_setstatus(&service) == 0);
	assert(test_vsvc_getstatus(&service) == 0);
	assert(test_vsvc_setactive(&service) == 0);
	assert(test_vsvc_setinactive(&service) == 0);
	assert(test_vsvc_markup(&service) == 0);
	assert(test_vsvc_markdown(&service) == 0);

	printf("vsvc ok\n");
	
	return(0);
}
